@@ -8,7 +8,7 @@ use Params::Check;
 use List::Util qw(first);
 use Carp;
 
-our $VERSION = '0.54';
+our $VERSION = '0.55';
 $Params::Check::WARNINGS_FATAL = 1;
 $Params::Check::CALLER_DEPTH   = $Params::Check::CALLER_DEPTH + 1;
 my $_attributes_made = {};
@@ -20,7 +20,7 @@ sub DEBUG { defined $_[1] ? ($DEBUG = $_[1]) : $DEBUG }
 
 #tablename
 sub TABLE {
-  croak("You must define a tablename for your class: sub TABLE {'tablename'}!");
+  croak("You must define a table-name for your class: sub TABLE {'tablename'}!");
 }
 
 #table columns
@@ -44,6 +44,67 @@ sub PRIMARY_KEY {'id'}
 
 sub ALIASES { {} }
 
+my $SQL_CACHE = {};
+my $SQL       = {};
+my $SQL       = {
+  SELECT => sub {
+    my $class = shift;
+    return $SQL_CACHE->{$class}{SELECT} ||= do {
+      my $where = $class->WHERE;
+      'SELECT ' 
+        . join(',', @{$class->COLUMNS}) 
+        . ' FROM ' 
+        . $class->TABLE
+        . (
+        (keys %$where)
+        ? ' WHERE '
+          . join(' AND ',
+          map { "$_=" . $class->dbix->dbh->quote($where->{$_}) }
+            keys %$where)
+        : ''
+        );
+      }
+  },
+  BY_PK => sub {
+    my $class = $_[0];
+
+    #cache this query and return it
+    return $SQL_CACHE->{$class}{BY_PK} ||= do {
+      'SELECT '
+        . join(',', @{$class->COLUMNS})
+        . ' FROM '
+        . $class->TABLE
+        . ' WHERE '
+        . $class->PRIMARY_KEY . '=?';
+    };
+  },
+};
+
+sub SQL {
+  my ($class, $args) = _get_obj_args(@_);    #class
+  croak('This is a class method. Do not use as object method.') if ref $class;
+  if (ref $args) {                           #adding new SQL strings($k=>$v pairs)
+    return $SQL->{$class} = {%{$SQL->{$class} || {}}, %{$args || {}}};
+  }
+
+  #a key
+  if ($args && !ref $args) {
+
+    #allow subclasses to override parent sqls and cache produced SQL
+    my $_SQL = $SQL_CACHE->{$class}{$args} || $SQL->{$class}{$args} || $SQL->{$args};
+    if (ref $_SQL) {
+      return $_SQL->($class);
+    }
+    else {
+      return $_SQL;
+    }
+  }
+
+  #they want all
+  return $SQL;
+}
+
+
 my $DBIX;    #DBIx::Simple instance
 
 #ATTRIBUTES
@@ -79,6 +140,11 @@ sub query {
   return dbix->query(@_)->object($class);
 }
 
+sub select_by_pk {
+  my $class = shift;
+  return dbix->query($SQL_CACHE->{$class}{BY_PK} || $class->SQL('BY_PK'), shift)
+    ->object($class);
+}
 
 sub _make_field_attrs {
   my $class = shift;
@@ -122,6 +188,7 @@ SUB
 
 #for outside modification during tests
 sub _attributes_made {$_attributes_made}
+sub _SQL_CACHE       {$SQL_CACHE}
 
 #conveninece for getting key/vaule arguments
 sub _get_args {
@@ -181,12 +248,12 @@ sub save {
 }
 
 sub update {
-  my ($self)  = @_;
-  my $pk      = $self->PRIMARY_KEY;
+  my ($self) = @_;
+  my $pk = $self->PRIMARY_KEY;
   $self->{data}{$pk} || croak('Please define primary key column (\$self->$pk(?))!');
   my @columns = @{$self->COLUMNS};
-  my $SET = join(', ', map {qq($/$_=?)} @columns);
-  my $SQL = sprintf(
+  my $SET     = join(', ', map {qq($/$_=?)} @columns);
+  my $SQL     = sprintf(
     'UPDATE ' . $self->TABLE . " SET $SET WHERE $pk=%s",
     dbix->{dbh}->quote($self->{data}{$pk})
   );
@@ -198,7 +265,7 @@ sub insert {
   my ($pk, $table, @columns) = ($self->PRIMARY_KEY, $self->TABLE, @{$self->COLUMNS});
   my $SQL =
       "INSERT INTO $table ("
-    . join(',', map {$_} @columns)
+    . join(',', @columns)
     . ') VALUES('
     . join(',', map {'?'} @columns) . ')';
   dbix->query($SQL, (map { $self->{data}{$_} } @columns));
@@ -222,7 +289,7 @@ sub insert {
 #--regex-perl=/^=head2\s+(.+)/-- \1/p,pod,Plain Old Documentation/
 #--regex-perl=/^=head[3-5]\s+(.+)/---- \1/p,pod,Plain Old Documentation/
 
-__END__
+#__END__
 
 =encoding utf8
 
@@ -232,7 +299,7 @@ DBIx::Simple::Class - Advanced object construction for DBIx::Simple!
 
 =head1 DESCRIPTION
 
-This module is writen to replace most of the abstraction stuff from the base 
+This module is written to replace most of the abstraction stuff from the base 
 model class in the MYDLjE project on github, but can be used independently as well. 
 
 The class provides some useful methods which simplify representing rows from 
@@ -241,7 +308,7 @@ at all. It does not support relational mapping. This is left to the developers u
 
 DBIx::Simple::Class is a database table/row abstraction. 
 At the same time it is not just a fancy representation of a table row 
-like DBIx::Simple::Result::RowObject (well you could nmake your subclass which is :)). 
+like DBIx::Simple::Result::RowObject (well you could make your subclass which is :)). 
 See below for details.
 Last but not least, this module has no other non-CORE dependencies besides DBIx::Simple.
 
@@ -269,7 +336,7 @@ Last but not least, this module has no other non-CORE dependencies besides DBIx:
   
   1;#end of My::Model::AdminUser
 
-  #2. In a startup script or subroutine
+  #2. In a start-up script or subroutine
   DBIx::Simple::Class->dbix( DBIx::Simple->connect(...) );
 
   #3. usage 
@@ -311,7 +378,7 @@ Last but not least, this module has no other non-CORE dependencies besides DBIx:
 
 You B<must> define it in your subclass. This is the table where 
 your object will store its data. Must return a string - the table name. 
-And with little imaginaniton you could put here some complex SQL or 
+And with little imagination you could put here some complex SQL or 
 an already prepared view: 
 
   (SELECT * FROM users WHERE column1='something' column2='other')
@@ -333,14 +400,14 @@ This constant is optional.
   sub WHERE { {data_type => 'note',published=>1 } };
   #...
   use My::PublishedNote;
-  #somwhere in your app
+  #somewhere in your application
   my $note = My::PublishedNote->select(id=>12345);
                                                       
 =head2 COLUMNS
 
 You B<must> define it in your subclass. 
-It must return an ARRAYREF with table columns to which the data is written.
-It is used  in L<DBIx::Simple/select> when retreiving a row from the database 
+It must return an ARRAY-REF with table columns to which the data is written.
+It is used  in L<DBIx::Simple/select> when retrieving a row from the database 
 and when saving object data. This list is also used to generate specific 
 getters and setters for each data-field.
 
@@ -349,11 +416,12 @@ getters and setters for each data-field.
   dbix->select($class->TABLE, $class->COLUMNS, {%{$class->WHERE}, %$where})->object($class);
 
 In case you have table columns that collide with some of the methods defined in this class like L</data>,
-L</save> etc., you can define aliases that will be used as method names. See </ALIASES>.
+L</save> etc., you can define aliases that will be used as method names. 
+See L</ALIASES>.
 
 =head2 CHECKS
 
-You B<must> define this soubroutine/constant in your class and put in it your
+You B<must> define this subroutine/constant in your class and put in it your
 C<$_CHECKS>. 
 C<$_CHECKS> is a HASHREF that must conform to the syntax supported by L<Params::Check/Template>.
 
@@ -373,7 +441,7 @@ in the same table. Default: 'id'.
 In case you have table columns that collide with some of the package methods like L</data>,
 L</save> etc., you can define aliases that will be used as method names. 
 
-You are free to define your own getters/setter for fields. They will not be overriden. 
+You are free to define your own getters/setter for fields. They will not be overridden. 
 All they need to do is to check the validity of the input and put the changed value in 
 C<$self-E<gt>{data}>.
 
@@ -419,7 +487,7 @@ to the current DBIx::Simple instance (with L<SQL::Abstract> support eventually :
 
 =head2 DEBUG
 
-Flag to enable/disable debug warnings. Influencess all DBIx::Simple::Class subclasses.
+Flag to enable/disable debug warnings. Influences all DBIx::Simple::Class subclasses.
 
     DBIx::Simple::Class->DEBUG(1);
     my $note = My::Note->new;# see in the log what methods are generated for your columns
@@ -467,18 +535,28 @@ statement. Prepends the L</WHERE> clause defined by you to the parameters.
 If a row is found, puts it in L</data>. 
 Returns an instance of your class on success or C<undef> otherwise.
 
-  my $user = MYDLjE::M::User->select(id => $user_id);
+    # Build your WHERE using an SQL::Abstract structure:
+    my $user = MYDLjE::M::User->select(id => $user_id);
 
 =head2 query
 
 A convenient wrapper for C<$dbix-E<gt>query($SQL,@bind)-E<gt>object($class)> and constructor. 
-Accepts exacttly the same arguments as L<DBIx::Simple/query>.
+Accepts exactly the same arguments as L<DBIx::Simple/query>.
 Returns an instance of your class on success or C<undef> otherwise.
 
   my $user = My::User->query(
     'SELECT ' . join (',',My::User->COLUMNS)
     . ' FROM ' . My::User->TABLE.' WHERE id=? and disabled=?', 12345, 0);
 
+=head2 select_by_pk
+
+Retreives  a row from the L</TABLE> by L</PRIMARY_KEY>. 
+Returns an instance of your class on success or C<undef> otherwise.
+
+    my $user = My::User->select_by_pk(1234);
+
+
+
 =head2 data
 
 Common getter/setter for all L</COLUMNS>. 
@@ -494,10 +572,10 @@ Returns a HASHREF - name/value pairs of the fields.
 
 Intelligent saver. If the object is fresh 
 ( not instantiated via L</new_from_dbix_simple> and L</select>) prepares and 
-executes an C<INSERT> statment, otherwise preforms an C<UPDATE>. 
+executes an C<INSERT> statement, otherwise preforms an C<UPDATE>. 
 L</TABLE> and L</COLUMNS> are used to construct the SQL. 
 L</data> is stored as a row in L</TABLE>.
-Returns the value of the internally performed opperation. See below.
+Returns the value of the internally performed operation. See below.
 
   my $note = MyNote->new(title=>'My Title', description =>'This is a great story.');
   #do something more...
@@ -516,7 +594,7 @@ See L<DBIx::Simple/last_insert_id>.
 =head2 update
 
 Used internally in L</save>. Can be used when you are sure your object is 
-retreived from the database. Returns true on success.
+retrieved from the database. Returns true on success.
 
   use My::Model::AdminUser;
   my $user = $dbix->query(
@@ -525,9 +603,44 @@ retreived from the database. Returns true on success.
   $user->first_name('Fred')->last_name('Flintstone');
   $user->update;
 
+=head2 SQL
+
+A getter/setter for custom SQL code. 
+
+Class method. 
+You can add key/value pairs in your class and then use them in your application.
+The values can be simple strings or subroutine references.
+There are two entries in this base class that you can use as example 
+implementations. Look at the source for details.
+The subroutine references are executed/evaluated only once and their output is 
+cached for performance.
+
+  package My::SiteUser;
+  use base qw(My::User);#a subclass of DBIx::Simple::Class
+  sub WHERE { {disabled => 0, group_id => 2} }
+  
+  #these could be very complex and retreived from a file where you keep them!
+  __PACKAGE__->SQL(
+    GUEST => 'SELECT * FROM users WHERE login_name = \'guest\'',
+    DISABLED => sub{
+        'SELECT * FROM'.__PACKAGE__->TABLE.' WHERE disabled=?';
+    }
+    LAST_N_REGISTERED => __PACKAGE__->SQL('SELECT')
+        .' order by id desc LIMIT ?, ?'
+  );
+
+  1;
+  # in your application
+  $SU ='My::SiteUser';
+  my $guest = $SU->query($SU->SQL('GUEST'));
+  my @members = $SU->query($SU->SQL('SELECT'));#allll ;)
+  my @disabled = $SU->query($SU->SQL('DISABLED'), 1);
+  my @enabled = $SU->query($SU->SQL('DISABLED'), 0);
+
 =head1 EXAMPLES
 
-Please look at the test file C<t/01-dbix-simple-class.t> of the distribution for a welth of examples.
+Please look at the test file C<t/01-dbix-simple-class.t> of the distribution 
+for a wealth of examples.
 
 
 =head1 AUTHOR
@@ -581,13 +694,15 @@ L<http://search.cpan.org/dist/DBIx-Simple-Class/>
 
 =head1 SEE ALSO
 
-L<DBIx::Simple>, L<DBIx::Simple::Result::RowObject>, L<DBIx::Simple::OO> L<SQL::Abstract>, L<Params::Check>
+L<DBIx::Simple>, L<DBIx::Simple::Result::RowObject>, L<DBIx::Simple::OO> 
+L<DBIx::Class>, L<Data::ObjectDriver>,L<Class::DBI>, L<Class::DBI::Lite>, 
+L<SQL::Abstract>, L<Params::Check>
 L<https://github.com/kberov/MYDLjE>
 
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012 Красимир Беров.
+Copyright 2012 Красимир Беров (Krasimir Berov).
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
